@@ -3,7 +3,7 @@
  * scram-common.h
  *		Declarations for helper functions used for SCRAM authentication
  *
- * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/common/scram-common.h
@@ -35,14 +35,14 @@
 #define SCRAM_RAW_NONCE_LEN			18
 
 /*
- * Length of salt when generating new verifiers, in bytes.  (It will be stored
+ * Length of salt when generating new secrets, in bytes.  (It will be stored
  * and sent over the wire encoded in Base64.)  16 bytes is what the example in
  * RFC 7677 uses.
  */
 #define SCRAM_DEFAULT_SALT_LEN		16
 
 /*
- * Default number of iterations when generating verifier.  Should be at least
+ * Default number of iterations when generating secret.  Should be at least
  * 4096 per RFC 7677.
  */
 #define SCRAM_DEFAULT_ITERATIONS	4096
@@ -66,7 +66,7 @@ extern void scram_H(const uint8 *str, int len, uint8 *result);
 extern void scram_ClientKey(const uint8 *salted_password, uint8 *result);
 extern void scram_ServerKey(const uint8 *salted_password, uint8 *result);
 
-extern char *scram_build_verifier(const char *salt, int saltlen, int iterations,
-								  const char *password);
+extern char *scram_build_secret(const char *salt, int saltlen, int iterations,
+								const char *password);
 
 #endif							/* SCRAM_COMMON_H */
