@@ -17,9 +17,9 @@
  */
 
 /* connstring parsing */
-bool parse_database(void *base, const char *name, const char *connstr);
+bool parse_database(void *base, const char *name, const char *connstr) _MUSTCHECK;
 
-bool parse_user(void *base, const char *name, const char *params);
+bool parse_user(void *base, const char *name, const char *params) _MUSTCHECK;
 
 /* user file parsing */
 bool load_auth_file(const char *fn)  /* _MUSTCHECK */;
