@@ -36,6 +36,7 @@ PgSocket *compare_connections_by_time(PgSocket *lhs, PgSocket *rhs);
 bool evict_connection(PgDatabase *db)		_MUSTCHECK;
 bool evict_user_connection(PgUser *user)	_MUSTCHECK;
 bool find_server(PgSocket *client)		_MUSTCHECK;
+bool life_over(PgSocket *server);
 bool release_server(PgSocket *server)		/* _MUSTCHECK */;
 bool finish_client_login(PgSocket *client)	_MUSTCHECK;
 bool check_fast_fail(PgSocket *client)		_MUSTCHECK;
@@ -53,9 +54,9 @@ PgUser * force_user(PgDatabase *db, const char *username, const char *passwd) _M
 PgUser * add_pam_user(const char *name, const char *passwd) _MUSTCHECK;
 
 void accept_cancel_request(PgSocket *req);
-void forward_cancel_request(PgSocket *server);
+bool forward_cancel_request(PgSocket *server);
 
-void launch_new_connection(PgPool *pool);
+void launch_new_connection(PgPool *pool, bool evict_if_needed);
 
 bool use_client_socket(int fd, PgAddr *addr, const char *dbname, const char *username, uint64_t ckey, int oldfd, int linkfd,
 		       const char *client_end, const char *std_string, const char *datestyle, const char *timezone,
