@@ -16,6 +16,9 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <usual/logging.h>
+#include <usual/string.h>
+
 /*
  * logging about specific socket
  */
