@@ -100,15 +100,17 @@ bool socket_set_keepalive(int fd, int onoff, int keepidle, int keepintvl, int ke
  *
  * Supports: ipv4, ipv5, unix sockets.
  */
-const char *sa2str(const struct sockaddr *sa, char *buf, int buflen);
+const char *sa2str(const struct sockaddr *sa, char *buf, size_t buflen);
 
 #ifndef HAVE_INET_NTOP
+#undef inet_ntop
 #define inet_ntop(a,b,c,d) usual_inet_ntop(a,b,c,d)
 /** Compat: inet_ntop() */
 const char *inet_ntop(int af, const void *src, char *dst, int cnt);
 #endif
 
 #ifndef HAVE_INET_PTON
+#undef inet_pton
 #define inet_pton(a,b,c) usual_inet_pton(a,b,c)
 /** Compat: inet_pton() */
 int inet_pton(int af, const char *src, void *dst);
