@@ -411,7 +411,7 @@ static PgDatabase *find_or_register_database(PgSocket *admin, const char *name)
 		db = register_auto_database(name);
 		if (db != NULL) {
 			slog_info(admin,
-			          "registered new auto-database: %s", name);
+				  "registered new auto-database: %s", name);
 		}
 	}
 	return db;
@@ -588,8 +588,8 @@ static bool admin_show_users(PgSocket *admin, const char *arg)
 	return true;
 }
 
-#define SKF_STD "sssssisiTTiiissis"
-#define SKF_DBG "sssssisiTTiiissisiiiiiii"
+#define SKF_STD "sssssisiTTiiississ"
+#define SKF_DBG "sssssisiTTiiississiiiiiii"
 
 static void socket_header(PktBuf *buf, bool debug)
 {
@@ -599,6 +599,7 @@ static void socket_header(PktBuf *buf, bool debug)
 				    "connect_time", "request_time",
 				    "wait", "wait_us", "close_needed",
 				    "ptr", "link", "remote_pid", "tls",
+				    "application_name",
 				    /* debug follows */
 				    "recv_pos", "pkt_pos", "pkt_remain",
 				    "send_pos", "send_remain",
@@ -618,6 +619,8 @@ static void socket_row(PktBuf *buf, PgSocket *sk, const char *state, bool debug)
 	char l_addr[PGADDR_BUF], r_addr[PGADDR_BUF];
 	IOBuf *io = sk->sbuf.io;
 	char infobuf[96] = "";
+	VarCache *v = &sk->vars;
+	const struct PStr *application_name = v->var_list[VAppName];
 	usec_t now = get_cached_time();
 	usec_t wait_time = sk->query_start ? now - sk->query_start : 0;
 
@@ -659,6 +662,7 @@ static void socket_row(PktBuf *buf, PgSocket *sk, const char *state, bool debug)
 			     (int)(wait_time % USEC),
 			     sk->close_needed,
 			     ptrbuf, linkbuf, remote_pid, infobuf,
+			     application_name ? application_name->str : "",
 			     /* debug */
 			     io ? io->recv_pos : 0,
 			     io ? io->parse_pos : 0,
@@ -698,6 +702,8 @@ static bool admin_show_clients(PgSocket *admin, const char *arg)
 
 		show_socket_list(buf, &pool->active_client_list, "active", false);
 		show_socket_list(buf, &pool->waiting_client_list, "waiting", false);
+		show_socket_list(buf, &pool->active_cancel_req_list, "active_cancel_req", false);
+		show_socket_list(buf, &pool->waiting_cancel_req_list, "waiting_cancel_req", false);
 	}
 
 	admin_flush(admin, buf, "SHOW");
@@ -725,6 +731,8 @@ static bool admin_show_servers(PgSocket *admin, const char *arg)
 		show_socket_list(buf, &pool->used_server_list, "used", false);
 		show_socket_list(buf, &pool->tested_server_list, "tested", false);
 		show_socket_list(buf, &pool->new_server_list, "new", false);
+		show_socket_list(buf, &pool->active_cancel_server_list, "active_cancel", false);
+		show_socket_list(buf, &pool->being_canceled_server_list, "being_canceled", false);
 	}
 	admin_flush(admin, buf, "SHOW");
 	return true;
@@ -819,11 +827,15 @@ static bool admin_show_pools(PgSocket *admin, const char *arg)
 		admin_error(admin, "no mem");
 		return true;
 	}
-	pktbuf_write_RowDescription(buf, "ssiiiiiiiiiis",
+	pktbuf_write_RowDescription(buf, "ssiiiiiiiiiiiiis",
 				    "database", "user",
 				    "cl_active", "cl_waiting",
-				    "cl_cancel_req",
-				    "sv_active", "sv_idle",
+				    "cl_active_cancel_req",
+				    "cl_waiting_cancel_req",
+				    "sv_active",
+				    "sv_active_cancel",
+				    "sv_being_canceled",
+				    "sv_idle",
 				    "sv_used", "sv_tested",
 				    "sv_login", "maxwait",
 				    "maxwait_us", "pool_mode");
@@ -832,12 +844,15 @@ static bool admin_show_pools(PgSocket *admin, const char *arg)
 		waiter = first_socket(&pool->waiting_client_list);
 		max_wait = (waiter && waiter->query_start) ? now - waiter->query_start : 0;
 		pool_mode = pool_pool_mode(pool);
-		pktbuf_write_DataRow(buf, "ssiiiiiiiiiis",
+		pktbuf_write_DataRow(buf, "ssiiiiiiiiiiiiis",
 				     pool->db->name, pool->user->name,
 				     statlist_count(&pool->active_client_list),
 				     statlist_count(&pool->waiting_client_list),
-				     statlist_count(&pool->cancel_req_list),
+				     statlist_count(&pool->active_cancel_req_list),
+				     statlist_count(&pool->waiting_cancel_req_list),
 				     statlist_count(&pool->active_server_list),
+				     statlist_count(&pool->active_cancel_server_list),
+				     statlist_count(&pool->being_canceled_server_list),
 				     statlist_count(&pool->idle_server_list),
 				     statlist_count(&pool->used_server_list),
 				     statlist_count(&pool->tested_server_list),
@@ -1316,7 +1331,7 @@ static bool admin_show_help(PgSocket *admin, const char *arg)
 		"\tRECONNECT [<db>]\n"
 		"\tKILL <db>\n"
 		"\tSUSPEND\n"
-		"\tSHUTDOWN\n",
+		"\tSHUTDOWN\n"
 		"\tWAIT_CLOSE [<db>]", "");
 	if (res)
 		res = admin_ready(admin, "SHOW");
