@@ -106,8 +106,12 @@ static bool send_client_authreq(PgSocket *client)
 		return false;
 	}
 
-	if (!res)
+	if (!res) {
+	    slog_noise(client, "No authentication response received");
 		disconnect_client(client, false, "failed to send auth req");
+	} else {
+		slog_noise(client, "Auth request sent successfully");
+	}
 	return res;
 }
 
@@ -145,7 +149,8 @@ static void start_auth_query(PgSocket *client, const char *username)
 		 */
 	}
 	if (!res)
-		disconnect_server(client->link, false, "unable to send login query");
+		disconnect_server(client->link, false, "unable to send auth_query");
+	client->expect_rfq_count++;
 }
 
 static bool login_via_cert(PgSocket *client)
@@ -195,7 +200,7 @@ static bool finish_set_pool(PgSocket *client, bool takeover)
 	bool ok = false;
 	int auth;
 
-	if (!client->login_user->mock_auth) {
+	if (!client->login_user->mock_auth && !client->db->fake) {
 		PgUser *pool_user;
 
 		if (client->db->forced_user)
@@ -222,9 +227,6 @@ static bool finish_set_pool(PgSocket *client, bool takeover)
 		}
 	}
 
-	if (!check_fast_fail(client))
-		return false;
-
 	if (takeover)
 		return true;
 
@@ -288,20 +290,13 @@ bool set_pool(PgSocket *client, const char *dbname, const char *username, const
 	client->db = find_database(dbname);
 	if (!client->db) {
 		client->db = register_auto_database(dbname);
-		if (!client->db) {
-			disconnect_client(client, true, "no such database: %s", dbname);
-			if (cf_log_connections)
-				slog_info(client, "login failed: db=%s user=%s", dbname, username);
-			return false;
-		} else {
+		if (client->db)
 			slog_info(client, "registered new auto-database: db=%s", dbname);
-		}
 	}
-
-	/* are new connections allowed? */
-	if (client->db->db_disabled) {
-		disconnect_client(client, true, "database does not allow connections: %s", dbname);
-		return false;
+	if (!client->db) {
+		client->db = calloc(1, sizeof(*client->db));
+		client->db->fake = true;
+		strlcpy(client->db->name, dbname, sizeof(client->db->name));
 	}
 
 	if (client->db->admin) {
@@ -362,12 +357,16 @@ bool set_pool(PgSocket *client, const char *dbname, const char *username, const
 					client->db->auth_user = add_user(cf_auth_user, "");
 			}
 			if (client->db->auth_user) {
-				if (takeover) {
-					client->login_user = add_db_user(client->db, username, password);
-					return finish_set_pool(client, takeover);
+				if (client->db->fake)
+					slog_debug(client, "not running auth_query because database is fake");
+				else {
+					if (takeover) {
+						client->login_user = add_db_user(client->db, username, password);
+						return finish_set_pool(client, takeover);
+					}
+					start_auth_query(client, username);
+					return false;
 				}
-				start_auth_query(client, username);
-				return false;
 			}
 
 			slog_info(client, "no such user: %s", username);
@@ -394,7 +393,7 @@ bool handle_auth_query_response(PgSocket *client, PktHdr *pkt) {
 			return false;
 		}
 		if (columns != 2u) {
-			disconnect_server(server, false, "expected 2 columns from login query, not %hu", columns);
+			disconnect_server(server, false, "expected 2 columns from auth_query, not %hu", columns);
 			return false;
 		}
 		break;
@@ -405,7 +404,7 @@ bool handle_auth_query_response(PgSocket *client, PktHdr *pkt) {
 			return false;
 		}
 		if (columns != 2u) {
-			disconnect_server(server, false, "expected 2 columns from login query, not %hu", columns);
+			disconnect_server(server, false, "expected 2 columns from auth_query, not %hu", columns);
 			return false;
 		}
 		if (!mbuf_get_uint32be(&pkt->data, &length)) {
@@ -413,7 +412,7 @@ bool handle_auth_query_response(PgSocket *client, PktHdr *pkt) {
 			return false;
 		}
 		if (length == (uint32_t)-1) {
-			disconnect_server(server, false, "login query response contained null user name");
+			disconnect_server(server, false, "auth_query response contained null user name");
 			return false;
 		}
 		if (!mbuf_get_chars(&pkt->data, length, &username)) {
@@ -490,8 +489,11 @@ bool handle_auth_query_response(PgSocket *client, PktHdr *pkt) {
 		if (server->state == SV_FREE || server->state == SV_JUSTFREE)
 			return false;
 		return true;
+	case 'E':	/* ErrorResponse */
+		disconnect_server(server, false, "error response from auth_query");
+		return false;
 	default:
-		disconnect_server(server, false, "unexpected response from login query");
+		disconnect_server(server, false, "unexpected response from auth_query");
 		return false;
 	}
 	sbuf_prepare_skip(&server->sbuf, pkt->len);
@@ -806,7 +808,7 @@ static bool handle_client_startup(PgSocket *client, PktHdr *pkt)
 					return false;
 				if (scram_client_final(client, length, data)) {
 					/* save SCRAM keys for user */
-					if (!client->scram_state.adhoc) {
+					if (!client->scram_state.adhoc && !client->db->fake) {
 						memcpy(client->pool->user->scram_ClientKey,
 						       client->scram_state.ClientKey,
 						       sizeof(client->scram_state.ClientKey));
@@ -924,7 +926,7 @@ static bool handle_client_work(PgSocket *client, PktHdr *pkt)
 
 	/* client wants to go away */
 	default:
-		slog_error(client, "unknown pkt from client: %d/0x%x", pkt->type, pkt->type);
+		slog_error(client, "unknown pkt from client: %u/0x%x", pkt->type, pkt->type);
 		disconnect_client(client, true, "unknown pkt");
 		return false;
 	case 'X': /* Terminate */
@@ -1013,7 +1015,7 @@ bool client_proto(SBuf *sbuf, SBufEvent evtype, struct MBuf *data)
 					  hdr2hex(data, hex, sizeof(hex)));
 			return false;
 		}
-		slog_noise(client, "read pkt='%c' len=%d", pkt_desc(&pkt), pkt.len);
+		slog_noise(client, "read pkt='%c' len=%u", pkt_desc(&pkt), pkt.len);
 
 		/*
 		 * If we are reading an SSL request or GSSAPI
