@@ -177,8 +177,8 @@ static void per_loop_activate(PgPool *pool)
 	int sv_tested, sv_used;
 
 	/* if there is a cancel request waiting, open a new connection */
-	if (!statlist_empty(&pool->cancel_req_list)) {
-		launch_new_connection(pool);
+	if (!statlist_empty(&pool->waiting_cancel_req_list)) {
+		launch_new_connection(pool, /* evict_if_needed= */ true);
 		return;
 	}
 
@@ -191,7 +191,7 @@ static void per_loop_activate(PgPool *pool)
 
 			/* db not fully initialized after reboot */
 			if (client->wait_for_welcome && !pool->welcome_msg_ready) {
-				launch_new_connection(pool);
+				launch_new_connection(pool, /* evict_if_needed= */ true);
 				continue;
 			}
 
@@ -206,7 +206,7 @@ static void per_loop_activate(PgPool *pool)
 			--sv_used;
 		} else {
 			/* not enough connections */
-			launch_new_connection(pool);
+			launch_new_connection(pool, /* evict_if_needed= */ true);
 			break;
 		}
 	}
@@ -404,6 +404,17 @@ static void pool_client_maint(PgPool *pool)
 				disconnect_client(client, true, "query_wait_timeout");
 			}
 		}
+		statlist_for_each_safe(item, &pool->waiting_cancel_req_list, tmp) {
+			client = container_of(item, PgSocket, head);
+			Assert(client->state == CL_WAITING_CANCEL);
+			age = now - client->request_time;
+
+			if (cf_query_timeout > 0 && age > cf_query_timeout) {
+				disconnect_client(client, false, "cancel request query_timeout");
+			} else if (cf_query_wait_timeout > 0 && age > cf_query_wait_timeout) {
+				disconnect_client(client, false, "cancel request query_wait_timeout");
+			}
+		}
 	}
 
 	/* apply client_login_timeout to clients waiting for welcome pkt */
@@ -487,7 +498,7 @@ static void check_pool_size(PgPool *pool)
 	    pool_client_count(pool) > 0)
 	{
 		log_debug("launching new connection to satisfy min_pool_size");
-		launch_new_connection(pool);
+		launch_new_connection(pool, /* evict_if_needed= */ false);
 	}
 }
 
@@ -674,9 +685,13 @@ void kill_pool(PgPool *pool)
 
 	close_client_list(&pool->active_client_list, reason);
 	close_client_list(&pool->waiting_client_list, reason);
-	close_client_list(&pool->cancel_req_list, reason);
+
+	close_client_list(&pool->active_cancel_req_list, reason);
+	close_client_list(&pool->waiting_cancel_req_list, reason);
 
 	close_server_list(&pool->active_server_list, reason);
+	close_server_list(&pool->active_cancel_server_list, reason);
+	close_server_list(&pool->being_canceled_server_list, reason);
 	close_server_list(&pool->idle_server_list, reason);
 	close_server_list(&pool->used_server_list, reason);
 	close_server_list(&pool->tested_server_list, reason);
