@@ -23,6 +23,10 @@
 #include "bouncer.h"
 #include "scram.h"
 
+#include <usual/err.h>
+#include <usual/safeio.h>
+#include <usual/slab.h>
+
 /* those items will be allocated as needed, never freed */
 STATLIST(user_list);
 STATLIST(database_list);
@@ -165,8 +169,11 @@ void change_client_state(PgSocket *client, SocketState newstate)
 	case CL_ACTIVE:
 		statlist_remove(&pool->active_client_list, &client->head);
 		break;
-	case CL_CANCEL:
-		statlist_remove(&pool->cancel_req_list, &client->head);
+	case CL_ACTIVE_CANCEL:
+		statlist_remove(&pool->active_cancel_req_list, &client->head);
+		break;
+	case CL_WAITING_CANCEL:
+		statlist_remove(&pool->waiting_cancel_req_list, &client->head);
 		break;
 	default:
 		fatal("bad cur client state: %d", client->state);
@@ -194,8 +201,11 @@ void change_client_state(PgSocket *client, SocketState newstate)
 	case CL_ACTIVE:
 		statlist_append(&pool->active_client_list, &client->head);
 		break;
-	case CL_CANCEL:
-		statlist_append(&pool->cancel_req_list, &client->head);
+	case CL_ACTIVE_CANCEL:
+		statlist_append(&pool->active_cancel_req_list, &client->head);
+		break;
+	case CL_WAITING_CANCEL:
+		statlist_append(&pool->waiting_cancel_req_list, &client->head);
 		break;
 	default:
 		fatal("bad new client state: %d", client->state);
@@ -223,12 +233,18 @@ void change_server_state(PgSocket *server, SocketState newstate)
 	case SV_TESTED:
 		statlist_remove(&pool->tested_server_list, &server->head);
 		break;
+	case SV_BEING_CANCELED:
+		statlist_remove(&pool->being_canceled_server_list, &server->head);
+		break;
 	case SV_IDLE:
 		statlist_remove(&pool->idle_server_list, &server->head);
 		break;
 	case SV_ACTIVE:
 		statlist_remove(&pool->active_server_list, &server->head);
 		break;
+	case SV_ACTIVE_CANCEL:
+		statlist_remove(&pool->active_cancel_server_list, &server->head);
+		break;
 	default:
 		fatal("bad old server state: %d", server->state);
 	}
@@ -254,6 +270,9 @@ void change_server_state(PgSocket *server, SocketState newstate)
 	case SV_TESTED:
 		statlist_append(&pool->tested_server_list, &server->head);
 		break;
+	case SV_BEING_CANCELED:
+		statlist_append(&pool->being_canceled_server_list, &server->head);
+		break;
 	case SV_IDLE:
 		if (server->close_needed || cf_server_round_robin) {
 			/* try to avoid immediate usage then */
@@ -266,6 +285,9 @@ void change_server_state(PgSocket *server, SocketState newstate)
 	case SV_ACTIVE:
 		statlist_append(&pool->active_server_list, &server->head);
 		break;
+	case SV_ACTIVE_CANCEL:
+		statlist_append(&pool->active_cancel_server_list, &server->head);
+		break;
 	default:
 		fatal("bad server state: %d", server->state);
 	}
@@ -508,7 +530,10 @@ static PgPool *new_pool(PgDatabase *db, PgUser *user)
 	statlist_init(&pool->tested_server_list, "tested_server_list");
 	statlist_init(&pool->used_server_list, "used_server_list");
 	statlist_init(&pool->new_server_list, "new_server_list");
-	statlist_init(&pool->cancel_req_list, "cancel_req_list");
+	statlist_init(&pool->waiting_cancel_req_list, "waiting_cancel_req_list");
+	statlist_init(&pool->active_cancel_req_list, "active_cancel_req_list");
+	statlist_init(&pool->active_cancel_server_list, "active_cancel_server_list");
+	statlist_init(&pool->being_canceled_server_list, "being_canceled_server_list");
 
 	list_append(&user->pool_list, &pool->map_head);
 
@@ -598,7 +623,7 @@ bool check_fast_fail(PgSocket *client)
 		return true;
 
 	/* Else we fail the client. */
-	disconnect_client(client, true, "pgbouncer cannot connect to server");
+	disconnect_client(client, true, "server login has been failing, try again later (server_login_retry)");
 
 	/*
 	 * Try to launch a new connection.  (launch_new_connection()
@@ -607,7 +632,7 @@ bool check_fast_fail(PgSocket *client)
 	 * clients, so we need to do it here to get any new servers
 	 * eventually.
 	 */
-	launch_new_connection(pool);
+	launch_new_connection(pool, /* evict_if_needed= */ true);
 
 	return false;
 }
@@ -715,7 +740,7 @@ static bool reset_on_release(PgSocket *server)
 	return res;
 }
 
-static bool life_over(PgSocket *server)
+bool life_over(PgSocket *server)
 {
 	PgPool *pool = server->pool;
 	usec_t lifetime_kill_gap = 0;
@@ -726,6 +751,11 @@ static bool life_over(PgSocket *server)
 	if (age < cf_server_lifetime)
 		return false;
 
+	/*
+	 * Calculate the time that disconnects because of server_lifetime
+	 * must be separated.  This avoids the need to re-launch lot
+	 * of connections together.
+	 */
 	if (pool_pool_size(pool) > 0)
 		lifetime_kill_gap = cf_server_lifetime / pool_pool_size(pool);
 
@@ -740,14 +770,18 @@ bool release_server(PgSocket *server)
 {
 	PgPool *pool = server->pool;
 	SocketState newstate = SV_IDLE;
+	struct List *cancel_item, *tmp;
 
 	Assert(server->ready);
 
 	/* remove from old list */
 	switch (server->state) {
+	case SV_BEING_CANCELED:
 	case SV_ACTIVE:
-		server->link->link = NULL;
-		server->link = NULL;
+		if (server->link) {
+			server->link->link = NULL;
+			server->link = NULL;
+		}
 
 		if (*cf_server_reset_query && (cf_server_reset_query_always ||
 					       pool_pool_mode(pool) == POOL_SESSION))
@@ -773,9 +807,23 @@ bool release_server(PgSocket *server)
 		fatal("bad server state: %d", server->state);
 	}
 
+	statlist_for_each_safe(cancel_item, &server->canceling_clients, tmp) {
+		PgSocket *cancel_client = container_of(cancel_item, PgSocket, cancel_head);
+		/*
+		 * If the cancel request is not in flight yet we can simply unlink
+		 * the cancel_client. When a cancel request doesn't have a
+		 * canceled_server linked to it forward_cancel_request will simply drop
+		 * the cancel request without forwarding it anywhere.
+		 */
+		if (cancel_client->state == CL_WAITING_CANCEL) {
+			cancel_client->canceled_server = NULL;
+			statlist_remove(&server->canceling_clients, cancel_item);
+		}
+	}
+
 	/* enforce lifetime immediately on release */
 	if (server->state != SV_LOGIN && life_over(server)) {
-		disconnect_server(server, true, "server_lifetime");
+		disconnect_server(server, true, "server lifetime over");
 		pool->last_lifetime_disconnect = get_cached_time();
 		return false;
 	}
@@ -786,6 +834,11 @@ bool release_server(PgSocket *server)
 		return false;
 	}
 
+	if (statlist_count(&server->canceling_clients) > 0) {
+		change_server_state(server, SV_BEING_CANCELED);
+		return true;
+	}
+
 	Assert(server->link == NULL);
 	slog_noise(server, "release_server: new state=%d", newstate);
 	change_server_state(server, newstate);
@@ -800,16 +853,24 @@ bool release_server(PgSocket *server)
 	return true;
 }
 
-/* drop server connection */
-void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
+/*
+ * close server connection
+ *
+ * send_term=true means to send a Terminate message to the server
+ * before disconnecting, send_term=false means to disconnect without.
+ * The latter is for protocol and communication errors where a normal
+ * protocol termination is not possible.
+ */
+void disconnect_server(PgSocket *server, bool send_term, const char *reason, ...)
 {
-	PgPool *pool = server->pool;
-	PgSocket *client;
-	static const uint8_t pkt_term[] = {'X', 0,0,0,4};
-	bool send_term = true;
 	usec_t now = get_cached_time();
 	char buf[128];
 	va_list ap;
+	struct List *cancel_item, *tmp;
+
+	if (server == NULL) {
+		return;
+	}
 
 	va_start(ap, reason);
 	vsnprintf(buf, sizeof(buf), reason, ap);
@@ -821,14 +882,27 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 			  (now - server->connect_time) / USEC);
 
 	switch (server->state) {
-	case SV_ACTIVE:
-		client = server->link;
+	case SV_ACTIVE_CANCEL:
+	case SV_ACTIVE:	{
+		PgSocket *client = server->link;
+
 		if (client) {
 			client->link = NULL;
 			server->link = NULL;
-			disconnect_client(client, true, "%s", reason);
+			/*
+			 * Send reason to client if it is already
+			 * logged in, otherwise send generic message.
+			 */
+			if (client->state == CL_ACTIVE || client->state == CL_WAITING)
+				disconnect_client(client, true, "%s", reason);
+			else if (client->state == CL_ACTIVE_CANCEL)
+				disconnect_client(client, false, "successfully sent cancel request");
+			else
+				disconnect_client(client, true, "bouncer config error");
 		}
+
 		break;
+	}
 	case SV_TESTED:
 	case SV_USED:
 	case SV_IDLE:
@@ -840,8 +914,8 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 		 */
 		if (!server->ready)
 		{
-			pool->last_login_failed = true;
-			pool->last_connect_failed = true;
+			server->pool->last_login_failed = true;
+			server->pool->last_connect_failed = true;
 		}
 		else
 		{
@@ -850,7 +924,7 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 			 * cancellation, so to the best of our knowledge we can connect to
 			 * the server, reset last_connect_failed accordingly.
 			 */
-			pool->last_connect_failed = false;
+			server->pool->last_connect_failed = false;
 			send_term = false;
 		}
 		break;
@@ -860,8 +934,15 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 
 	Assert(server->link == NULL);
 
+	statlist_for_each_safe(cancel_item, &server->canceling_clients, tmp) {
+		PgSocket *cancel_client = container_of(cancel_item, PgSocket, cancel_head);
+		cancel_client->canceled_server = NULL;
+		statlist_remove(&server->canceling_clients, cancel_item);
+	}
+
 	/* notify server and close connection */
-	if (send_term && notify) {
+	if (send_term) {
+		static const uint8_t pkt_term[] = {'X', 0,0,0,4};
 		bool _ignore = sbuf_answer(&server->sbuf, pkt_term, sizeof(pkt_term));
 		(void) _ignore;
 	}
@@ -881,7 +962,14 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 		log_noise("sbuf_close failed, retry later");
 }
 
-/* drop client connection */
+/*
+ * close client connection
+ *
+ * notify=true means to send the reason message as an error to the
+ * client, notify=false means no message is sent.  The latter is for
+ * protocol and communication errors where sending a regular error
+ * message is not possible.
+ */
 void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 {
 	usec_t now = get_cached_time();
@@ -930,16 +1018,42 @@ void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 				release_server(server);
 			}
 		}
+		break;
+	case CL_ACTIVE_CANCEL:
+	case CL_WAITING_CANCEL:
+		/*
+		 * If the cancel request is still linked to the server that it
+		 * cancelled (or wanted to cancel) a query from, this is the time to
+		 * unlink them. The cancel request has finished at this point and we're
+		 * going to free its memory soon, so we don't want references to it
+		 * left behind.
+		 */
+		if (client->canceled_server) {
+			PgSocket *canceled_server = client->canceled_server;
+			statlist_remove(&canceled_server->canceling_clients, &client->cancel_head);
+			client->canceled_server = NULL;
+
+			/*
+			 * If the linked server was waiting until all cancel requests
+			 * targeting it were finished, and we were the last cancel request,
+			 * then we can now safely move the server to the idle state. We
+			 * trigger this by calling release_server again.
+			 */
+			if (canceled_server->state == SV_BEING_CANCELED
+					&& statlist_count(&canceled_server->canceling_clients) == 0 ) {
+				release_server(canceled_server);
+			}
+		}
+		break;
 	case CL_WAITING:
 	case CL_WAITING_LOGIN:
-	case CL_CANCEL:
 		break;
 	default:
 		fatal("bad client state: %d", client->state);
 	}
 
 	/* send reason to client */
-	if (notify && reason && client->state != CL_CANCEL) {
+	if (notify && reason && client->state != CL_WAITING_CANCEL) {
 		/*
 		 * don't send Ready pkt here, or client won't notice
 		 * closed connection
@@ -952,6 +1066,10 @@ void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 		free(client->login_user);
 		client->login_user = NULL;
 	}
+	if (client->db && client->db->fake) {
+		free(client->db);
+		client->db = NULL;
+	}
 
 	change_client_state(client, CL_JUSTFREE);
 	if (!sbuf_close(&client->sbuf))
@@ -1026,19 +1144,41 @@ static void dns_connect(struct PgSocket *server)
 	struct sockaddr_in6 sa_in6;
 	struct sockaddr *sa;
 	struct PgDatabase *db = server->pool->db;
-	const char *host = db->host;
-	const char *unix_dir;
+	const char *host;
 	int sa_len;
 	int res;
+	char *host_copy = NULL;
+
+	/* host list? */
+	if (db->host && strchr(db->host, ',')) {
+		int count = 1;
+		int n;
+
+		for (const char *p = db->host; *p; p++)
+			if (*p == ',')
+				count++;
+
+		host_copy = xstrdup(db->host);
+		for (host = strtok(host_copy, ","), n = 0; host; host = strtok(NULL, ","), n++)
+			if (server->pool->rrcounter % count == n)
+				break;
+		Assert(host);
+
+		server->pool->rrcounter++;
+	} else {
+		host = db->host;
+	}
 
 	if (!host || host[0] == '/' || host[0] == '@') {
+		const char *unix_dir;
+
 		memset(&sa_un, 0, sizeof(sa_un));
 		sa_un.sun_family = AF_UNIX;
 		unix_dir = host ? host : cf_unix_socket_dir;
 		if (!unix_dir || !*unix_dir) {
 			log_error("unix socket dir not configured: %s", db->name);
 			disconnect_server(server, false, "cannot connect");
-			return;
+			goto cleanup;
 		}
 		snprintf(sa_un.sun_path, sizeof(sa_un.sun_path),
 			 "%s/.s.PGSQL.%d", unix_dir, db->port);
@@ -1058,18 +1198,18 @@ static void dns_connect(struct PgSocket *server)
 		sa = (struct sockaddr *)&sa_un;
 		res = 1;
 	} else if (strchr(host, ':')) {  /* assume IPv6 address on any : in addr */
-		slog_noise(server, "inet6 socket: %s", db->host);
+		slog_noise(server, "inet6 socket: %s", host);
 		memset(&sa_in6, 0, sizeof(sa_in6));
 		sa_in6.sin6_family = AF_INET6;
-		res = inet_pton(AF_INET6, db->host, &sa_in6.sin6_addr);
+		res = inet_pton(AF_INET6, host, &sa_in6.sin6_addr);
 		sa_in6.sin6_port = htons(db->port);
 		sa = (struct sockaddr *)&sa_in6;
 		sa_len = sizeof(sa_in6);
 	} else { /* else try IPv4 */
-		slog_noise(server, "inet socket: %s", db->host);
+		slog_noise(server, "inet socket: %s", host);
 		memset(&sa_in, 0, sizeof(sa_in));
 		sa_in.sin_family = AF_INET;
-		res = inet_pton(AF_INET, db->host, &sa_in.sin_addr);
+		res = inet_pton(AF_INET, host, &sa_in.sin_addr);
 		sa_in.sin_port = htons(db->port);
 		sa = (struct sockaddr *)&sa_in;
 		sa_len = sizeof(sa_in);
@@ -1078,15 +1218,17 @@ static void dns_connect(struct PgSocket *server)
 	/* if simple parse failed, use DNS */
 	if (res != 1) {
 		struct DNSToken *tk;
-		slog_noise(server, "dns socket: %s", db->host);
+		slog_noise(server, "dns socket: %s", host);
 		/* launch dns lookup */
-		tk = adns_resolve(adns, db->host, dns_callback, server);
+		tk = adns_resolve(adns, host, dns_callback, server);
 		if (tk)
 			server->dns_token = tk;
-		return;
+		goto cleanup;
 	}
 
 	connect_server(server, sa, sa_len);
+cleanup:
+	free(host_copy);
 }
 
 PgSocket *compare_connections_by_time(PgSocket *lhs, PgSocket *rhs)
@@ -1150,13 +1292,29 @@ bool evict_user_connection(PgUser *user)
 	return false;
 }
 
-/* the pool needs new connection, if possible */
-void launch_new_connection(PgPool *pool)
+/*
+ * Launches a new connection if possible.
+ *
+ * Called when the pool needs new connection.
+ *
+ * If `evict_if_needed` is true and the db or user has reached their
+ * connection limits, this method will attempt to evict existing connections
+ * from other users/dbs to make room for the new connection.
+ */
+void launch_new_connection(PgPool *pool, bool evict_if_needed)
 {
 	PgSocket *server;
 	int max;
 
-	/* allow only small number of connection attempts at a time */
+	/*
+	 * Allow only a single connection attempt at a time.
+	 *
+	 * NOTE: If this is ever changed to allow more than a single connection
+	 * attempt at once (which would probably be a good thing), some code needs
+	 * to change that depends on the fact that there's only ever one connection
+	 * attempt at once. At least a little bit below in this function, where
+	 * connections are opened for cancel requests.
+	 */
 	if (!statlist_empty(&pool->new_server_list)) {
 		log_debug("launch_new_connection: already progress");
 		return;
@@ -1174,8 +1332,16 @@ void launch_new_connection(PgPool *pool)
 
 	max = pool_server_count(pool);
 
-	/* when a cancel request is queued allow connections up to twice the pool size */
-	if (!statlist_empty(&pool->cancel_req_list) && max < (2 * pool_pool_size(pool))) {
+	/*
+	 * When a cancel request is queued allow connections up to twice the pool
+	 * size.
+	 *
+	 * NOTE: This logic might seem a bit confusing, because it seems like we'll
+	 * open many connections even if there's only a single cancel request. But
+	 * this works just fine, because we only ever open a single connection at
+	 * once (see top of this function).
+	 */
+	if (!statlist_empty(&pool->waiting_cancel_req_list) && max < (2 * pool_pool_size(pool))) {
 		log_debug("launch_new_connection: bypass pool limitations for cancel request");
 		goto force_new;
 	}
@@ -1202,7 +1368,7 @@ allow_new:
 	max = database_max_connections(pool->db);
 	if (max > 0) {
 		/* try to evict unused connections first */
-		while (pool->db->connection_count >= max) {
+		while (evict_if_needed && pool->db->connection_count >= max) {
 			if (!evict_connection(pool->db)) {
 				break;
 			}
@@ -1217,7 +1383,7 @@ allow_new:
 	max = user_max_connections(pool->user);
 	if (max > 0) {
 		/* try to evict unused connection first */
-		while (pool->user->connection_count >= max) {
+		while (evict_if_needed && pool->user->connection_count >= max) {
 			if (!evict_user_connection(pool->user)) {
 				break;
 			}
@@ -1241,6 +1407,7 @@ force_new:
 	server->pool = pool;
 	server->login_user = server->pool->user;
 	server->connect_time = get_cached_time();
+	statlist_init(&server->canceling_clients, "canceling_clients");
 	pool->last_connect_time = get_cached_time();
 	change_server_state(server, SV_LOGIN);
 	pool->db->connection_count++;
@@ -1286,6 +1453,18 @@ PgSocket *accept_client(int sock, bool is_unix)
 /* client managed to authenticate, send welcome msg and accept queries */
 bool finish_client_login(PgSocket *client)
 {
+	if (client->db->fake) {
+		if (cf_log_connections)
+			slog_info(client, "login failed: db=%s user=%s", client->db->name, client->login_user->name);
+		disconnect_client(client, true, "no such database: %s", client->db->name);
+		return false;
+	}
+
+	if (client->db->db_disabled) {
+		disconnect_client(client, true, "database \"%s\" is disabled", client->db->name);
+		return false;
+	}
+
 	switch (client->state) {
 	case CL_LOGIN:
 		change_client_state(client, CL_ACTIVE);
@@ -1303,7 +1482,7 @@ bool finish_client_login(PgSocket *client)
 		client->wait_for_welcome = true;
 		pause_client(client);
 		if (cf_pause_mode == P_NONE)
-			launch_new_connection(client->pool);
+			launch_new_connection(client->pool, /* evict_if_needed= */ true);
 		return false;
 	}
 	client->wait_for_welcome = false;
@@ -1317,7 +1496,10 @@ bool finish_client_login(PgSocket *client)
 	return true;
 }
 
-/* client->cancel_key has requested client key */
+/*
+ * Accepts a cancellation request, which will eventual cancel the query running
+ * on the client that matches req->client_key
+ */
 void accept_cancel_request(PgSocket *req)
 {
 	struct List *pitem, *citem;
@@ -1326,7 +1508,7 @@ void accept_cancel_request(PgSocket *req)
 
 	Assert(req->state == CL_LOGIN);
 
-	/* find real client this is for */
+	/* find the client that has the same cancel_key as this request */
 	statlist_for_each(pitem, &pool_list) {
 		pool = container_of(pitem, PgPool, head);
 		statlist_for_each(citem, &pool->active_client_list) {
@@ -1352,49 +1534,84 @@ found:
 		return;
 	}
 
-	/* not linked client, just drop it then */
-	if (!main_client->link) {
-		/* let administrative cancel be handled elsewhere */
-		if (main_client->pool->db->admin) {
-			disconnect_client(req, false, "cancel request for console client");
-			admin_handle_cancel(main_client);
-			return;
-		}
+	/*
+	 * cancel requests for administrative databases should be handled
+	 * differently from cancel request for normal servers. We should handle
+	 * these directly instead of forwarding them.
+	 */
+	if (main_client->pool->db->admin) {
+		disconnect_client(req, false, "cancel request for console client");
+		admin_handle_cancel(main_client);
+		return;
+	}
 
+	/*
+	 * The client is not linked to a server, which means that no query is
+	 * running that can be cancelled. This likely means the query finished by
+	 * itself before the cancel request arived to pgbouncer.
+	 */
+	if (!main_client->link) {
 		disconnect_client(req, false, "cancel request for idle client");
 
 		return;
 	}
 
-	/* drop the connection, if fails, retry later in justfree list */
-	if (!sbuf_close(&req->sbuf))
-		log_noise("sbuf_close failed, retry later");
-
-	/* remember server key */
+	/*
+	 * Link the cancel request and the server on which the query is being
+	 * cancelled in a many-to-one way.
+	 */
 	server = main_client->link;
-	memcpy(req->cancel_key, server->cancel_key, 8);
+	req->canceled_server = server;
+	statlist_append(&server->canceling_clients, &req->cancel_head);
 
-	/* attach to target pool */
+	/*
+	 * Attach to the target pool and change state to waiting_cancel. This way
+	 * once a new connection is opened, it's used to forward the cancel
+	 * request.
+	 */
 	req->pool = pool;
-	change_client_state(req, CL_CANCEL);
+	change_client_state(req, CL_WAITING_CANCEL);
 
-	/* need fresh connection */
-	launch_new_connection(pool);
+	/*
+	 * Open a new connection over which the cancel request is forwarded to the
+	 * server.
+	 */
+	launch_new_connection(pool, /* evict_if_needed= */ true);
 }
 
-void forward_cancel_request(PgSocket *server)
+bool forward_cancel_request(PgSocket *server)
 {
 	bool res;
-	PgSocket *req = first_socket(&server->pool->cancel_req_list);
+	PgSocket *req = first_socket(&server->pool->waiting_cancel_req_list);
 
-	Assert(req != NULL && req->state == CL_CANCEL);
+	Assert(req != NULL && req->state == CL_WAITING_CANCEL);
 	Assert(server->state == SV_LOGIN);
 
-	SEND_CancelRequest(res, server, req->cancel_key);
-	if (!res)
-		log_warning("sending cancel request failed: %s", strerror(errno));
+	/*
+	 * In between accepting the cancel request and receiving an open connection
+	 * the query that was supposed to be cancelled has now completed. This
+	 * becomes a problem when the server is then reused for some other client.
+	 * Because this will mean that the cancel that is forwarded will cancel a
+	 * query from a completely different client than the client it was intended
+	 * for.
+	 */
+	if (!req->canceled_server) {
+		disconnect_client(req, false, "not sending cancel request for client that is now idle");
+		return false;
+	}
 
-	change_client_state(req, CL_JUSTFREE);
+	server->link = req;
+	req->link = server;
+
+	SEND_CancelRequest(res, server, req->canceled_server->cancel_key);
+	if (!res) {
+		log_warning("sending cancel request failed: %s", strerror(errno));
+		disconnect_client(req, false, "failed to send cancel request");
+		return false;
+	}
+	log_info("started sending cancel request");
+	change_client_state(req, CL_ACTIVE_CANCEL);
+	return true;
 }
 
 bool use_client_socket(int fd, PgAddr *addr,
@@ -1527,6 +1744,7 @@ bool use_server_socket(int fd, PgAddr *addr,
 	server->login_user = user;
 	server->connect_time = server->request_time = get_cached_time();
 	server->query_start = 0;
+	statlist_init(&server->canceling_clients, "canceling_clients");
 
 	fill_remote_addr(server, fd, pga_is_unix(addr));
 	fill_local_addr(server, fd, pga_is_unix(addr));
