@@ -26,6 +26,9 @@
 
 #include "bouncer.h"
 
+#include <usual/safeio.h>
+#include <usual/slab.h>
+
 #ifdef USUAL_LIBSSL_FOR_TLS
 #define USE_TLS
 #endif
@@ -520,6 +523,7 @@ try_more:
 
 	if (sbuf->dst->sock == 0) {
 		log_error("sbuf_send_pending: no dst sock?");
+		sbuf_call_proto(sbuf, SBUF_EV_SEND_FAILED);
 		return false;
 	}
 
@@ -620,7 +624,7 @@ static void sbuf_try_resync(SBuf *sbuf, bool release)
 	IOBuf *io = sbuf->io;
 
 	if (io) {
-		log_noise("resync(%d): done=%d, parse=%d, recv=%d",
+		log_noise("resync(%d): done=%u, parse=%u, recv=%u",
 			  sbuf->sock,
 			  io->done_pos, io->parse_pos, io->recv_pos);
 	}
