@@ -27,6 +27,8 @@
 
 #include "bouncer.h"
 
+#include <usual/safeio.h>
+
 /*
  * Takeover done, old process shut down,
  * kick this one running.
@@ -365,7 +367,7 @@ void takeover_init(void)
 		fatal("no admin pool?");
 
 	log_info("takeover_init: launching connection");
-	launch_new_connection(pool);
+	launch_new_connection(pool, /* evict_if_needed= */ true);
 }
 
 void takeover_login_failed(void)
