@@ -38,7 +38,6 @@ pgctl() {
 
 ulimit -c unlimited
 
-# System configuration checks
 SED_ERE_OP='-E'
 case `uname` in
 Linux)
@@ -73,9 +72,14 @@ rm -rf $PGDATA
 if [ ! -d $PGDATA ]; then
 	echo "initdb"
 	mkdir $PGDATA
-	initdb --nosync >> $PG_LOG 2>&1
-	sed -r -i "/unix_socket_director/s:.*(unix_socket_director.*=).*:\\1 '/tmp':" pgdata/postgresql.conf
+	initdb -A trust --nosync >> $PG_LOG
+	echo "unix_socket_directories = '/tmp'" >> pgdata/postgresql.conf
 	echo "port = $PG_PORT" >> pgdata/postgresql.conf
+	# We need to make the log go to stderr so that the tests can
+	# check what is being logged.  This should be the default, but
+	# some packagings change the default configuration.
+	echo "logging_collector = off" >> pgdata/postgresql.conf
+	echo "log_destination = stderr" >> pgdata/postgresql.conf
 	echo "log_connections = on" >> pgdata/postgresql.conf
 	echo "log_disconnections = on" >> pgdata/postgresql.conf
 	cp pgdata/postgresql.conf pgdata/postgresql.conf.orig
